#!/usr/bin/python
# -*- coding: utf-8 -*-
# Class managing cells
# MW 2012 GPLv3+
"""
.. module:: ct_Cells
   :synopsis: This module aims to handle the cells, this is to say that it should contains methods allowing
   determination of contours, segmentation, tracking...
"""

import math, time, os, gobject, imp
mod_phases = imp.load_source('mod_phases', './modules/modules_core/mod_phases.py')
mod_fluorescence = imp.load_source('mod_fluorescence', './modules/modules_treatment/mod_fluorescence.py')
import numpy as np

list_wrappers = [('mod_cell_polygon', './modules/modules_cell/mod_cell_polygon.py'),
				('mod_cell_rectangle', './modules/modules_cell/mod_cell_rectangle.py')]

class Cells :
	"""This is the class managing cells, it provides various methods to add or delete cells.
	This class is instanciated by :class:`ct_Movies.Movie`, and contains a reference to it.
	
	:param movie: The :class:`ct_Movies.Movie` instance
	:type movie: :class:`ct_Movies.Movie`
	"""
	def __init__(self, movie) :
		self.movie = movie
		self.cell_wrappers = self._load_cell_wrappers()
		self.cells_list = []
		
	def _load_cell_wrappers(self) :
		"""Function that returns a dict of cell wrapper modules"""
		wrappers = {}
		for t in list_wrappers :
			m = imp.load_source(t[0], t[1])
			wrappers[m.get_name()] = m
			
		#print '----controls'
		#for key in wrappers :
		#	print wrappers[key].get_name()
		return wrappers
		
	def get_cell_wrappers_dict(self) :
		return self.cell_wrappers
		
	def get_save_dict(self) :
		ce_l = self.get_cells_list()
		l = []
		for ce in ce_l :
			dic = {}
			index = ce.get_index()
			save_dic = ce.get_save_dict()
			dic['index'] = {'desc' : 'index of the cell', 'value' : index}
			dic['cell_data'] = {'desc' : 'The internal parameters of the concerned cell', 'value' : save_dic}
			l.append(dic)
		out = {}
		out['cells'] = {'desc' : "A list of all the initiated cells, with their prameters", 'value' : l}
		return out
	def set_save_dict(self, dic) :
		ce_l = dic['cells']['value']
		for ce in ce_l :
			self.get_cells_list()[ce['index']['value']].set_save_dict(ce['cell_data']['value'])
	def add_cell(self, center_rect, size, angle) :
		"""Methods create and returns a :class:`ct_Cells` instance. This class contains a :class:`ct_Movies.Movie` instance
		
		.. note::
			For the moment, the cell is defined by a rectangle : its center and its size, 
			but since we want to handle non-square cells, this will change in the future
		
		:param center_rect: Coordinates of the center of the rectangle containing the cell. 
			Input format is a tuple of integer 
		:type center_rect: (:class:`int`, int)
		
		:param size: The size of the rectangle, as a tuple of two integers, the first one representing
			the size following the x axis, the second one following the y axis.
		:type size: (int, int)
		
		:param angle: The inclinaison of the defined rectangle. Angles are counted positively in the 
			trigonometric convention (anti-clockwise). Angles must be expressed in degrees.
		:type angle: int
			
		.. note::
			This function only accept integers as angles, but this should change in the future
			
		:returns: :class:`ct_Cells.Cell` instance
		:rtype: :class:`ct_Cells.Cell`
		"""
		c = Cell(self.movie, center_rect, size, angle)
		i = len(self.cells_list)
		c.set_index(i)
		self.cells_list.append(c)
		return c
		
	def get_cells_list(self) :
		"""Returns a list of the :class:`ct_Cells` instances
		
		:returns: list of :class:`ct_Cells.Cell` instances
		:rtype: list
		"""
		return self.cells_list
		
	def set_cells_list(self, lis) :
		"""Replace the list of cells by another list
		
		:param list: list of :class:`ct_Cells.Cell`
		:type lis: list
		"""
		self.cells_list = lis
	
	def clear_cells_list(self) :
		"""Clears the list of cells"""
		self.cells_list = []
		
class Cell :
	"""Class that represents a cell. Each cells contains a reference to the :class:`ct_Movies.Movie` instance
	
	.. note::
		For the moment, a cell is represented in a fixed rectangle. Thus, this is a very limited approximation,
		and this class will evolve in the future.
	
	:param movie: The :class:`ct_Movies.Movie` instance
	:type movie: :class:`ct_Movies.Movie`
	
	:param center_rect: Coordinates of the center of the rectangle containing the cell. 
		Input format is a tuple of integer 
	:type center_rect: (:class:`int`, int)
		
	:param size: The size of the rectangle, as a tuple of two integers, the first one representing
		the size following the x axis, the second one following the y axis.
	:type size: (int, int)
	
	:param angle: The inclinaison of the defined rectangle. Angles are counted positively in the 
		trigonometric convention (anti-clockwise). Angles must be expressed in degrees.
	:type angle: int
			
	.. note::
		This function only accept integers as angles, but this should change in the future
	
	.. warning:: This class is not suitable to perform any tracking
	"""
	
	def __init__(self, movie, center_rect, size, angle) :
		"""center_rect is the center of the rectangle that contains the cell
		NOT SUITABLE FOR TRACKING
		size is a tuple containing the dimensions of the rectangle (x_size, y_size)""" 
		self.cv = movie.movie_opencv.cv
		self.movie = movie
		self.center_rect = center_rect
		self.size = size	# Size seems to be the half sizes (after manipulations)
		self.angle = angle
		self._set_rect()
		self.font_init = False
		self.reference = None	# Reference polygon
		self.selected = False
		self.frame = None		# The frame index
		self.auto_toggle = False
		self.add_button_pos = 0
		self.phases = mod_phases.Phases(self, movie)
		self.fluorescence = mod_fluorescence.Fluorescence(self, self.movie)
	def get_save_dict(self) :
		dic = {}
		dic['selected'] = {'desc' : 'if the cell is selected or excluded', 'value' : self.is_selected()}
		dic['phases'] = {'desc' : 'Phases data', 'value' : self.phases.get_save_dict()}
		return dic
	def set_save_dict(self, dic) :
		self.selected = dic['selected']['value']
		self.phases.set_save_dict(dic['phases']['value'])
		print self.selected
	def set_frame(self, i) :
		"""Saves the given index as the index of the current frame.
		
		.. warning::
			At that time, no verification is performed, and you must check that the input index is a valid index
			
		:param i: The index of the frame
		:type i: int
		"""
		self.frame = int(i)
	
	def update_panel(self) :
		
		print 'old update'
		"""Function that updates the :guilabel:`Cell Panel` with the last data coming from
		the OpenCv player.
		
		.. note::
			For the moment, only the :guilabel:`phase` label is updated based on the :func:`_get_phase` function"""
		# Update phase button
		ph = self._get_phase()
		self.l_phase2.set_label(ph[0])
		color = self.gtk.gdk.color_parse(ph[1])
		style = self.l_phase2.get_style().copy()
		style.bg[self.gtk.STATE_NORMAL] = color
		style.bg[self.gtk.STATE_ACTIVE] = color
		style.bg[self.gtk.STATE_PRELIGHT] = color
		self.l_phase2.set_style(style)
		self.l_phase2.show()
		
		# Update select checkbox
		#self.auto_toggle = False
		#print (self.is_selected(), self.cb_select.get_active(), self.auto_toggle)
		if self.is_selected() != self.cb_select.get_active() and not self.auto_toggle :
			#print (self.is_selected(), self.cb_select.get_active())
			self.auto_toggle = True
			self.cb_select.set_active(self.is_selected())
	
	def _get_phase(self) :
		#### To WRITE properly
		return self.phases.get_phase()
	def is_cell(self) :
		return self.phases.is_cell()
	
	def set_ref(self, p) :
		"""This function is supposed to be initializing a reference polygon in order to perform background 
		subtraction/correction.
		
		.. note::
			In this version, the reference polygon is defined as the border of the defined cell (and is included
			in it, which means that the size of the cell includes in this version the size of the reference
			polygon.
			
		:param p: The width, in pixel of the border that will act as a reference zone.
		:type p: int
		"""
		s = self.size
		self.reference = self._calc_rect(self.center_rect, [s[0]-int(p), s[1]-int(p)])
		
	def _get_ref(self) :
		"""returns polygon reference"""
		return self.reference
	
	def _calc_rect(self, c, s) :
		"""Returns a rectangle with a specified center, size and angle"""
		b1 = math.atan(s[1]/float(s[0])) - self.angle*math.pi/180
		b2 = math.atan(s[1]/float(s[0])) + self.angle*math.pi/180
		l = (s[0]**2+s[1]**2)**0.5
		r = []
		r.append((int(c[0]-l*math.cos(b1)),int(c[1]-l*math.sin(b1))))
		r.append((int(c[0]+l*math.cos(b2)),int(c[1]-l*math.sin(b2))))
		r.append((int(c[0]+l*math.cos(b1)),int(c[1]+l*math.sin(b1))))
		r.append((int(c[0]-l*math.cos(b2)),int(c[1]+l*math.sin(b2))))
		
		return r
	def _set_rect(self) :
		self.rect = self._calc_rect(self.center_rect, self.size)
	
	###def get_rect(self) :
	def get_rect(self, best_fit=True, angle=None) :
		"""Returns the coordinates of the rectangle surrounding the cell as defined by
		when instanciating the :class:`ct_Cells.Cell` class.
		This rectangle is the rectangle that fits best the polygon if best_fit==True
		or a rotated rectangle by angle (degree)
		
		:returns: A tuple containing the coordinates of the four points defining the rectangle.
			The first point is the top-left one, when displayed with a null inclination (angle=0).
			Then the points are in clockwise order.
		:rtype: (int, int, int, int) """
		if best_fit :
			pass
			# use cv.FitEllipse2(points) -> Box2D
		elif angle != None :
			pass
			# return "self.rect"
		else :
			print 'get_best_rect function called without any instructions (angle or best_fit)'
		return self.rect
		
	###def draw_rect(self, cv, image) :
	def draw_cell(self, cv, image, draw_ref=True) :
		"""Draws the limits of the cell and the limits of the reference image on the image
		This requires a :class:`cv` instance. Colors can be managed by editing the code
		
		:param cv: A :class:`cv` instance. OpenCV must be in version > 2.0 (but I don't know which 
			version exactly.
		:type cv: :class:`cv`
		
		:param image: :class:`cv.CvArr` (or a :class:`cv.IplImage` can be required, I don't know, please
			convert as appropriate
		:type image: :class:`cv.CvArr`
		
		.. warning::
			The type of the 'image' parameter is not well defined. Be careful!"""
		self.cv = cv
		r=self.get_rect()
		if not self.font_init :
			font = cv.InitFont(cv.CV_FONT_HERSHEY_PLAIN, 1, 1) 	# Initializing font
			self.font = font
			self.font_init = True
		else :
			font = self.font
		cv.PolyLine(image, [list(r)], True, (255,255,0))
		cv.PutText(image, str(self.index), (r[0][0], r[0][1]+14), font, (255,255,0))
		if draw_ref :
			# Draw reference polygon
			s = self.size
			r = self._calc_rect(self.center_rect, [s[0]-15, s[1]-15])
			cv.PolyLine(image, [list(self._get_ref())], True, (255,255,255))
	def change_select(self, widget=None, data=None, state=None) :
		"""Function that switches between two states of the cell : selected and not selected
		The selected flag will be used when performing batch processing of the cell.
		
		.. note::
			This function is called when the select button is clicked, and the 'widget' and
			'data' parameters are *not* required to run the function.
			
		.. note::
			For the moment, the selected flag has no effect."""
		if not self.auto_toggle :	
			if state != None :
				self.selected = state
				#self.cb_select.set_active(state)
			else :
				self.selected = not self.selected
		
		self.auto_toggle = False

	def select(self, but=False) :
		"""Select the cell, if but==True, the checkbox is also updated"""
		"""if but :
			self.cb_select.set_active(True)"""
		self.selected = True
	
	def unselect(self, but=False) :
		"""Unselect the cell, if but==True, the checkbox is also updated"""
		"""
		if but :		# Check if instanciated/ to delete with the updater
			self.cb_select.set_active(False)"""
		self.selected = False
			
	def new_frame(self, gtk) :
		"""Creates and returns a panel containing all the controls for the management of the cell object
		It also initializes all the required mouse/button bindings.
		
		:param gtk: A :class:`gtk` instance
		:type gtk: :class:`gtk`
		
		:returns: A frame object that can be attached to a window.
		:rtype: :class:`gtk.Frame`"""
		self.gtk = gtk
		frame = self.gtk.Frame(str(self.get_index()))
		frame.show()
		table = self.gtk.Table(3,3)
		table.show()
		frame.add(table)
		
		# Labels
		l_phase = self.gtk.Label('Phase : ')
		l_phase.show()
		table.attach(l_phase, 0,1,0,1)
		
		ph = self._get_phase()
		self.l_phase2 = self.gtk.Button(str(self.get_index()))
		color = self.gtk.gdk.color_parse(ph[1])
		style = self.l_phase2.get_style().copy()
		style.bg[gtk.STATE_NORMAL] = color
		style.bg[gtk.STATE_ACTIVE] = color
		style.bg[gtk.STATE_PRELIGHT] = color
		self.l_phase2.set_style(style)
		self.l_phase2.show()
		table.attach(self.l_phase2, 1,2,0,1)	
		
		# Select button
		self.cb_select = self.gtk.CheckButton('Select')	###### init properly
		self.cb_select.connect("clicked", self.change_select, {})
		self.cb_select.show()
		table.attach(self.cb_select, 0, 1, 1, 2)
		
		# Buttons	
		b_find_phases = self._bouton(self.gtk, "Find phases", self._find_phases, {})
		b_find_phases.show()
		table.attach(b_find_phases, 1,2,1,2)
		
		b_configure_phases = self._bouton(self.gtk, "Configure...", self._test, {})
		b_configure_phases.show()
		table.attach(b_configure_phases, 2,3,1,2)
		
		# This button should be dynamically registered
		b_extract_fluo = self._bouton(self.gtk, "Extract fluorescence", self._extract_fluo, {})
		b_extract_fluo.show()
		table.attach(b_extract_fluo, 1,2,2,3)

		b_export = self._bouton(self.gtk, "Export", self._export, {})
		b_export.show()
		table.attach(b_export, 2,3,2,3)	
		self.table = table
		return frame
	
	def add_button(self, button) :
		pos = self.add_button_pos
		x_pos = pos % 3
		y_pos = pos//5
		self.table.resize(3+y_pos, 6)
		self.table.attach(button, 2+x_pos, 2+x_pos+1, 3+y_pos, 3+y_pos+1)
		self.add_button_pos += 1
	def _extract_fluo(self, w, data=None) :
		self.fluorescence.extract_fluo_GTK(self.cv, self.movie.grid.movie_gtk, True)
	def extract_fluo(self, w, index=None, obj=None) :
		self.fluorescence.extract_fluo_GTK(self.cv, self.movie.grid.movie_gtk, False, index, obj)
		
	def get_cell_without_reference(self, cell) :
		"""Returns the cell in numpy format, original shape is not conserved
		Takes the output of get_cell as input so that double computation is avoided
		Output is a numpy format given by channel"""
		cv = self.cv
		m_cv = cv.GetMat(cell)
		m_np = np.asarray(m_cv)
		s = m_np.shape		# This should be a three channels image, e.g :should return (512,512,3)
		ref_size = self.movie.grid.reference_size
		return m_np[ref_size:(s[0]-ref_size), ref_size:(s[1]-ref_size),:].astype('float32')#.reshape((-1,3))	## Take care of whom holds the border - in numpy the last element in the list is not reached (as range)
	def get_cell_without_reference_NP(self, cell) :
		"""Same as before but with a numpy array as input"""
		s = cell.shape
		ref_size = self.movie.grid.reference_size
		return cell[ref_size:(s[0]-ref_size), ref_size:(s[1]-ref_size),:].astype('float32')
		
	def get_cell_reference(self, cell) :
		"""Output in numpy format, original shape is not conserved"""
		cv = self.cv
		ref_size = self.movie.grid.reference_size
		m_cv = cv.GetMat(cell)
		m_np = np.asarray(m_cv)
		s = m_np.shape
		mask = np.zeros(s)				# Create mask
		mask[0:ref_size,:,:] =  m_np[0:ref_size,:,:]	# Fill mask
		mask[(s[0]-ref_size):s[0],:,:] =  m_np[(s[0]-ref_size):s[0],:,:]
		mask[:,0:ref_size,:] =  m_np[:,0:ref_size,:]
		mask[:,(s[1]-ref_size):s[1],:] =  m_np[:,(s[1]-ref_size):s[1],:] 
		#r_mask = m_np[:,:,0][mask==1]
		#g_mask = m_np[:,:,1][mask==1]
		#b_mask = m_np[:,:,2][mask==1]
		return mask.astype('float32')#m_np[mask==1].reshape((-1,3)) 	# Reshape mask (see get_cell_without_reference for the remark about reshaping - 3 channels)
		
	def get_cell_reference_NP(self, cell) :
		"""Output in numpy format, original shape is conserved"""
		ref_size = self.movie.grid.reference_size
		m_np =cell
		s = m_np.shape
		mask = np.zeros(s)				# Create mask
		mask[0:ref_size,:,:] =  m_np[0:ref_size,:,:]	# Fill mask
		mask[(s[0]-ref_size):s[0],:,:] =  m_np[(s[0]-ref_size):s[0],:,:]
		mask[:,0:ref_size,:] =  m_np[:,0:ref_size,:]
		mask[:,(s[1]-ref_size):s[1],:] =  m_np[:,(s[1]-ref_size):s[1],:] 
		return mask.astype('float32')#m_np[mask==1].reshape((-1,3)) 	# Reshape mask (see get_cell_without_reference for the remark about reshaping - 3 channels)
	
	def get_cell(self, channel=None, out_mode=8, format='cv') :
		"""Function that returns the current cell image (OpenCV format)"""
		cv = self.cv
		if out_mode == 8 :
			im2 = self.movie.get_current_frame_CV(channel)		# gets the image
		elif out_mode == 32 :
			im_in = self.movie.get_current_frame_32bits(channel)

			if im_in.channels != 3 :
				im_in2 = cv.CreateImage(cv.GetSize(im_in), cv.IPL_DEPTH_32F, 1)
				cv.ConvertScale(im_in,im_in2)
				im2 = cv.CreateImage(cv.GetSize(im_in2), im_in2.depth, 3)
				cv.CvtColor(im_in2, im2, cv.CV_GRAY2BGR)
			else :
				im2 = im_in
		rect_small = self.get_rect()		# Get all variables
		x_m = rect_small[0][0]
		x_M = rect_small[0][0]
		y_m = rect_small[0][1]
		y_M = rect_small[0][1]
		for p in rect_small :			# - surrounding rectangle
				if p[0] < x_m :
					x_m = p[0]
				elif p[0] > x_M :
					x_M = p[0]
				if p[1] < y_m :
					y_m = p[1]
				elif y_M < p[1] :
					y_M = p[1]
		a = self.angle
		s = self.size
		r = (x_m, y_m, x_M-x_m, y_M-y_m)
		#im2 = self.movie.movie_opencv.get_current_image()	# Working
		
		im = cv.CreateImage(cv.GetSize(im2), im2.depth, 3)
		cv.Copy(im2, im)
		cv.SetImageROI(im, r)
		
		mat = cv.CreateMat(2,3,cv.CV_32FC1)
		
		if im.depth == 8 :
			out = cv.CreateMat(r[3],r[2], cv.CV_8UC3)
		elif im.depth == 32 :
			out = cv.CreateMat(r[3],r[2], cv.CV_32FC3)
		cv.GetRotationMatrix2D((int(r[2]/2.0),int(r[3]/2.0)), -1*a, 1, mat)
		cv.WarpAffine(im, out, mat)
		s_x = rect_small[1][0]-rect_small[0][0]
		s_y = rect_small[2][1]-rect_small[1][1]
		if format == 'cv' :
			out = cv.GetImage(out)
			cv.SetImageROI(out, ((r[2]-s_x)/2, (r[3]-s_y)/2, s_x, s_y))
		if format == 'numpy' :
			out = np.asarray(out)
			"""print r
			print out.shape
			print (s_x,s_y)
			print '----'"""
			out = out[ ((r[3]-s_y)/2):((r[3]+s_y)/2),((r[2]-s_x)/2):((r[2]+s_x)/2) ]#.astype('float64')
		return out
		
	def _destroy_progress(self, data=None) :
		self.movie.grid.movie_gtk.b_set_grid.set_sensitive(True)
		self.dialog.destroy()
	def _bouton(self, gtk, label, function, dico) :
		"""Function that adds a button to the parent"""
		bouton = gtk.Button(label)			# Creation d'un nouveau bouton.
		bouton.connect("clicked", function, dico)	# On connecte le signal "clicked" du bouton a la fonction de rappel
		bouton.show()					# This packs the button into the window (a GTK container).
		return bouton
	
	def find_phases(self, warning) :
		"""If warning, the export requires the user to click Ok"""
		self.phases.find_phases_GTK(self.cv, self.movie.grid.movie_gtk, warning)
	
	def _find_phases(self, w, data=None) :
		"""Runs find_phases function in mod_phases module"""
		self.phases.find_phases_GTK(self.cv, self.movie.grid.movie_gtk)
	
	def export_cell(self, folder, warning=True, index=None, obj=None) :
		"""This function saves the cell in a specified folder. It performs the extraction of 
		the cell from the image, and its rotation to save it in a specified format.
		
		:param folder: The path to the folder where the images will be saved. This folder
			must exist and be empty.
		:type folder: str
		
		.. warning::
			This function contains a lot of warnings.
		
		.. warning::
			The code of this function is stil under development, it may not work properly.
		
		.. warning::
			The givent 'folder' must exist, else errors will be raised. Furthermore, this
			folder should be empty since the saving of the images will replace any file that
			has the same name as a saved image. *Be careful*.
		
		.. warning::
			This function locks the control panel since it requires to use the playback.
			
		.. warning::
				The :class:`ct_Cells.Cell` instancemust contain a reference to an instance 
				of the :class:`cv` module.
		
		"""
		gtk = self.movie.grid.movie_gtk.gtk
		#parent = self.movie.grid.window_process	# defined if draw_process_grid has been called
		self.movie.grid.hide_grid()			# Hide grid
		self.movie.grid.movie_gtk.b_set_grid.set_sensitive(False)	# Blocking grid editing
		
		self.dialog = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.dialog.connect("destroy", self._destroy_progress)
		
		table = gtk.Table(3,2, True)
		table.show()
		self.dialog.add(table)
		
		if warning :
			self.dialog.set_title("Please check playback options")
			label = gtk.Label("The current playback settings (speed, view mode)\n\
will be used to render the cell. Please correct them before clicking ok.\n\
Please note that for the moment the export runs from the beginning to the\n\
end of the video.")
			table.attach(label, 0, 2,0,1)	
			b_ok = self._bouton(gtk, 'Ok', self._run_export, folder, index, obj)
			table.attach(b_ok, 0,1,2,3)
			b_cancel = self._bouton(gtk, 'Cancel', self._destroy_progress, {})
			table.attach(b_cancel, 1,2,2,3)			
			
		else :
			self.dialog.set_title("Exporting cell")
			self._run_export('',folder, index, obj)
		
		# Create the ProgressBar
		self.pbar = gtk.ProgressBar()
		table.attach(self.pbar, 0,2,1,2)
		self.pbar.show()
		self.pbar.set_text('Press Ok when ready')
		
		self.dialog.show_all()
		self.frac = 0
		
	def _run_export(self, w, folder, index=None, obj=None) :
		print 'Running'
		self.frac = 0
		#self.movie.set_speed(1)				# Speed 1
		self.movie.set_mode('manual')		# Mode manual
		self.movie.set_frame_index(0)		# Set frame index
		self.movie.grid.movie_gtk.lock()	# Lock
		self.n = self.movie.get_nb_frames()
		self.running_export = False
		self.first_run = True
		
		txt = 'Save file : %s\nCell index : %s' % (str(self.movie.import_export.file), str(self.get_index()))
		f = open(folder + 'data.txt', 'w')
		f.write(txt)
		f.close()
		
		self.timer = gobject.timeout_add(100, self._export_timer, folder, index, obj)
			
	def _export_timer(self, folder, index=None, obj=None) :
		if self.first_run and obj.index==index :
			self.running_export = True
			self.frac = 0
			self.movie.set_speed(1)				# Speed 1
			self.movie.set_mode('manual')		# Mode manual
			self.movie.set_frame_index(0)		# Set frame index
			self.movie.grid.movie_gtk.lock()	# Lock
			self.movie.grid.hide_grid()	# Lock
			self.first_run = False
			self.progress_data = {'i' : 0}
			self.pbar.set_text('')

		cv = self.cv
		ret = True
		if self.running_export :
			i = self.progress_data['i']
			if i < self.n and obj.index==index :
				f = self.movie.get_frame_displayed()	# The frame currently displayed by opencv
				if f == i :
					#print (i, obj.index, index)
					path = folder + 'image'+str(i) + '.png'
														
					c = self.get_cell()					# Getting OpenCV image
					cv.SaveImage(path, c)
					self.movie.manual_next_frame = True
					self.movie.next_frame()
					self.pbar.set_text('image'+str(i) + '.png')
					if i%20 == 0 :
						print 'image'+str(i) + '.png'
					self.frac = float(i+1)/self.n
					self.pbar.set_fraction(self.frac)
					i+=1
				elif f>i :
					self.movie.set_frame_index(i)
			else :
				ret = False
				self.pbar.set_fraction(0)
				self.movie.grid.movie_gtk.unlock()	# Unlock
				self._destroy_progress()
				gobject.source_remove(self.timer)
				obj.index += 1
			self.progress_data['i'] = i
		return ret
	
	def is_selected(self) :
		return self.selected
			
	def _export(self, source=None, event=None) :
		"""Exports the cell as a sequence of images"""
		gtk = self.gtk
		chooser = gtk.FileChooserDialog(title=None,action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK))
		response = chooser.run()
		r = False
		if response == gtk.RESPONSE_OK:
			n = self.movie.get_name()
			t = time.strftime("%d-%m-%Y--%Hh%M", time.localtime())
			exp = chooser.get_filename()+'/'+str(n)+'-' + str(self.get_index()) + '-' + t + '/'
			os.mkdir(exp)
			r = True
		elif response == gtk.RESPONSE_CANCEL:
			pass
		chooser.destroy()
		print 'File chose'
		if r :
			self.export_cell(exp)	# Call export function (export the whole movie, gives it a folder URL

		
	def _test(self, source=None, event=None) :
		print 'Test... well !'			

	def is_point_in_cell(self, pt) :
		"""Returns if the point is in th cell"""
		r = self.get_rect()
		ret = self.cv.PointPolygonTest(r, pt, False)
		return ret
		
		
	def is_all_in_screen(self) :
		"""Returns 'True' if the cell is totally contained in the image
		
		:returns: Returns 'True' if the cell (and its reference) is totally contained in the
			image
		:rtype: bool"""
		pass
		r = self.get_rect()
		s = self.movie.get_size()
		out = True
		if not (r[0][0] >= 0 and r[0][1] >= 0 and r[1][0] < s[0] and r[1][1] >= 0 and r[2][0] < s[0] and r[2][1] < s[1] and r[3][0] > 0 and r[3][1] < s[1]) :
			out = False
		return out
	def set_index(self, i) :
		"""Sets the index of the cell. This is used by the :class:ct_Cells.Cells instance to
		give unique identifiers to the cells, which allows to display those indices
		
		:param i: The index of the cell to be set
		:type i: int
		
		.. note::
			No type/unicity is performed by the function, this should be handled before setting
			the index"""
		self.index = i	
	def get_index(self) :
		"""Returns the index of the cell
		
		:returns: Cell index
		:rtype: int"""
		return self.index
